#!/usr/bin/python3
# coding: utf-8

#####################################################################################
# Plot graphs with data given by operations                                         #
#                                                                                   #
# This script receives a file path as an argument and outputs, in the same folder   #
# as where the input is located, a bunch of plots                                   #
#                                                                                   #
# - number of requested nodes                                                       #
# - submit time of jobs relative to the first one                                   #
# - histogram with the number of processadors                                       #       
# - pie chart with the proporition of jobs per state                                #
#                                                                                   #
# DATA FROM OPERATIONS HAS TO BE PREPROCESSED TO VALID .CSV                         #
#                                                                                   #
# BSC-CNS - Earth Sciences                                                          #
# Manuel Giménez de Castro Marciani                                                 #
# 2022                                                                              #
#####################################################################################

import pandas as pd
import datetime
import matplotlib.pyplot as plt
import sys

# Number of NODES
def number_of_nodes(data, file_name, dir_path):
    print(f"Plot number of requested nodes histogram...")
    plt.hist(data["NNodes"], bins=100)
    plt.title(f"Distribution of Nodes requested\n Data from {file_name}")
    plt.ylabel("Number of Nodes")
    plt.xlabel("Number of Jobs")
    plt.savefig(f"{dir_path}/{file_name}_nodes.png")
    plt.clf()

# Number of PROCESSORS
def processors(data, file_name, dir_path):
    print(f"Plot number of requested cpus histogram...")
    plt.hist(data["NCPUS"], bins=100)
    plt.title(f"Distribution of CPUs requested\n Data from {file_name}")
    plt.ylabel("Number of CPUs")
    plt.xlabel("Number of Jobs")
    plt.savefig(f"{dir_path}/{file_name}_cpus.png")
    plt.clf()

# SUBMIT time
def submit_time(data, file_name, dir_path):
    print(f"Plot submit time histogram...")
    plt.hist(data["Zeroed submit"])
    plt.title(f"Distribution of submission time requested\n wrt to the first submission. Data from {file_name}")
    plt.ylabel("Number of Jobs")
    plt.xlabel("Submit time in seconds wrt to the first submission")
    plt.savefig(f"{dir_path}/{file_name}_submit_time.png")
    plt.clf()

# Number of jobs per STATE
def state_jobs(data, file_name, dir_path):
    print(f"Plot pie-chart with job status...")
    states = data["State"].unique()
    num_per_state = []
    for state in states:
        num_per_state.append(len(data[data["State"] == state]))
    plt.pie(num_per_state, labels=states)
    plt.title(f"Number of jobs per state\nData from {file_name}")
    plt.savefig(f"{dir_path}/{file_name}_states.png")
    plt.clf()

if __name__ == "__main__":

    if len(sys.argv) != 2:
        print("You have to provide the path to the CSV with COMMA AS DELIMITER")
        sys.exit(1)

    file_path = sys.argv[1]
    file_name = file_path.split("/")[-1].split(".")[0] # picks the file name without path and file extension
    dir_path = file_path.rsplit("/",1)[0] # remove the file name from directory path
    data = pd.read_csv(file_path, sep=",")

    print(f"Preprocessing {file_name}")

    # Transform submit data to datetime to make it easier to compute date differences
    formatted_date = []
    for i in range(len(data["Submit"])):
        formatted_date.append(datetime.datetime.strptime(data["Submit"][i], "%Y-%m-%dT%H:%M:%S"))
    # 8 is the position of the column, just for aesthetics 
    data.insert(8,"Formatted submit",formatted_date,allow_duplicates=True)

    print(f"Add formatted submit column")

    # this array will be the submit time wrt reference in the first submitted job
    reference = min(formatted_date) #pick the first job as the reference
    zeroed_date = []
    # We take as the reference the submission of the first job
    for i in range(len(data["Formatted submit"])):
        zeroed_date.append((data["Formatted submit"][i]-reference).seconds) # And take the time in seconds wrt to the reference 
    data.insert(9,"Zeroed submit", zeroed_date, allow_duplicates=True)

    print(f"Add zeroed submit column")

    # Transform elapsed format to seconds
    elapsed_in_seconds = []
    for i in range(len(data["Elapsed"])):
        current_data = data["Elapsed"][i].split(":")
        elapsed_in_seconds.append((int(current_data[0])*60 + int(current_data[1]))*60 + int(current_data[2]))
    data.insert(7, "Elapsed in seconds", elapsed_in_seconds, allow_duplicates=True)

    number_of_nodes(data, file_name, dir_path)
    submit_time(data, file_name, dir_path)
    processors(data, file_name, dir_path)
    state_jobs(data, file_name, dir_path)
