#!/usr/bin/python3
# coding: utf-8

#####################################################################################
# Slurm simulator's output trace multi-plot data script!                            #
#                                                                                   #
# This script receives a file path as an argument and outputs, in the same folder   # 
# as where the input file is located, a bunch of plots:                             #
#                                                                                   #
# - GLOBAL                                                                          #
#  - Start job time histogram                                                       #
#  - Number of nodes histogram                                                      #
#  - Time in queue histogram                                                        #
#  - 2d histogram of time in queue X number of nodes                                #
#  - Submit job time histogram                                                      #
#  - Evolution of machine usage (REEEEALLY SLOW CUZ IT IS A FOR IN PYTHON)          #
#- PER USER                                                                         #
#  - boxplot of start time job per user                                             #
#  - boxplot of requested nodes per user                                            #
#  - boxplot of wait time per user                                                  #
#  - Evolution of usage per user (REALLY SLOW because it is alsoo a for in python)  #
#                                                                                   #
# BSC-CNS - Earth Sciences                                                          #
# Manuel Giménez de Castro Marciani                                                 #
# 2022                                                                              #
#####################################################################################

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt 
import sys # To read input 

##### HARDCODED MACHINE PARAMETERS! #####
TOTAL_NODES = 3456 # MN4 
NO_CPUS_PER_NODE = 24
#########################################

## Time that the job started
def overall_start_time(data, file_name, dir_path):
    print("Plotting starting time histogram...")
    plt.hist(list(data["Start time"]))
    plt.title(f"Distribution of Start time\n Trace: {file_name}")
    plt.xlabel("Time in seconds [since the start of the simulation(???)]")
    plt.savefig(f"{dir_path}/{file_name}_hist.png")
    plt.clf()

## Number of requested nodes
def overall_requested_nodes(data, file_name, dir_path):
    print("Plotting requested nodes histogram...")
    plt.hist(list(data["Nodes"]))
    plt.title(f"Distribution of Nodes requested\n Trace: {file_name}")
    plt.xlabel("Number of nodes")
    plt.savefig(f"{dir_path}/{file_name}_nodes_hist.png")
    plt.clf()

## TIME IN QUEUE
def overall_time_in_queue(data, file_name, dir_path):
    print("Plotting time in queue histogram...")
    plt.hist(list(data["Wait time"]))
    plt.xlabel("Time in queue")
    plt.title(f"Time in Queue\n Trace: {file_name}")
    plt.savefig(f"{dir_path}/{file_name}_time_in_queue_hist.png")
    plt.clf()

## HISTOGRAM NUMBER OF NODES X WAIT TIME 
# this one is to check if usually bigger jobs have wait longer in queue (it is wait I expect)
def overall_2dhist_nodes_x_wait_time(data, file_name, dir_path):
    print("Plotting 2d histogram (requested nodes) X (wait time)...")
    plt.hist2d(list(data["Nodes"]), list(data["Wait time"]))
    plt.xlabel("Number of Nodes")
    plt.ylabel("Time in Queue (seconds)")
    plt.title(f"Time in queue per number of nodes\n Trace: {file_name}")
    plt.savefig(f"{dir_path}/{file_name}_nodes_time_in_queue_hist2d.png")
    plt.clf()

## SUBMIT TIME
def overall_submit_time(data, file_name, dir_path):
    print("Plotting submit time histogram...")
    plt.hist(list(data["Submit time"]))
    plt.title(f"Histogram of submit time\n Trace: {file_name}")
    plt.xlabel("Submit time in seconds [since the begining of simulation?]")
    plt.savefig(f"{dir_path}/{file_name}_submit_time_hist.png")
    plt.clf()

## EVOLUTION OF MACHINE TOTAL USAGE
def overall_evolution_machine_usage(data, file_name, dir_path):
    print("Plotting instant total machine usage...")
    # reference start is the starting time of the first job!
    # we call this time T
    start_time = np.min(data["Start time"])
    final_time = np.max(data["End time"])+1
    prop_nodes_per_time=[]
    # we go through every second from start_time and final_time and compute the total number of active processes
    for i in range(final_time):
        active_jobs = data[(data["Start time"] < i) & (data["End time"] >= i)]
        used_nodes = np.sum(active_jobs["Nodes"])
        prop_node = used_nodes/TOTAL_NODES #proportion of used nodes in T+i seconds
        # we multiple by 100 to have percentage!
        prop_nodes_per_time.append(100*prop_node)

    plt.plot(range(final_time),prop_nodes_per_time)
    plt.title(f"Instant machine-wide total usage\n Trace: {file_name}")
    plt.xlabel("Time in seconds")
    plt.ylabel("% of Usage")
    plt.savefig(f"{dir_path}/{file_name}_instant_usage.png")
    plt.clf()

#users[1001] = "user1/account1"
#users[1002] = "user2/account2"
#users[1003] = "tester/account1"

## starting time (this is kinda of stupid, isnt?)
def user_start_time(data, file_name, dir_path):
    print("Plotting start time per user...")
    users = data["UserId"].unique()
    start_time_per_user = []
    for user in users:
        start_time_per_user.append(data[data["UserId"] == user]["Start time"])
    plt.boxplot(start_time_per_user, labels=users)
    plt.title(f"Start time of jobs per user\n Trace: {file_name}")
    plt.ylabel("Time in seconds")
    plt.savefig(f"{dir_path}/{file_name}_start_time_boxplot.png")
    plt.clf()

## nodes requested per user
def user_nodes_requested(data, file_name, dir_path):
    print("Plotting nodes requested per user...")
    users = data["UserId"].unique()
    nodes_per_user = []
    for user in users:
        nodes_per_user.append(data[data["UserId"] == user]["Nodes"])
    plt.boxplot(nodes_per_user, labels=users)
    plt.title(f"Nodes requested per user\n Trace: {file_name}")
    plt.ylabel("Number of nodes")
    plt.savefig(f"{dir_path}/{file_name}_nodes_boxplot.png")
    plt.clf()

## wait time per user
def user_wait_time(data, file_name, dir_path):
    print("Plotting wait time per user...")
    users = data["UserId"].unique()
    wait_time_per_user = []
    for user in users:
        wait_time_per_user.append(data[data["UserId"] == user]["Wait time"])
    plt.boxplot(wait_time_per_user, labels=users)
    plt.title(f"Wait time of jobs per user\n Trace: {file_name}")
    plt.ylabel("Time in seconds")
    plt.savefig(f"{dir_path}/{file_name}_wait_time_boxplot.png")
    plt.clf()

## User usage EVOLUTION
def user_usage(data, file_name, dir_path):
    print("Plotting usage per user...")
    users = data["UserId"].unique()
    # Reference point is the first starting job
    start_time = min(data["Start time"])
    final_time = max(data["End time"])+1
    # array of arrays. Each of the arrays is the usage of the user in second i-th second from reference point start_time
    usage_evolution = []
    # COMPUTE INSTANT USAGE FOR EACH USER
    for user in users:
        user_jobs = data[data["UserId"] == user]
        user_usage_evolution = []
        # go through every second of the simulation
        for t in range(final_time):
            finished_jobs = user_jobs[user_jobs["End time"] < t]
            current_jobs = user_jobs[(user_jobs["Start time"] < t) & (user_jobs["End time"] >= t)]
            # RUNTIME*CORES is usage
            finished_usage = np.sum(finished_jobs["Run time"]*finished_jobs["Nodes"]*NO_CPUS_PER_NODE)
            current_usage = np.sum((t-current_jobs["Start time"])*current_jobs["Nodes"]*NO_CPUS_PER_NODE)
            user_usage_evolution.append(finished_usage+current_usage)
        usage_evolution.append(user_usage_evolution)

    # PLOT FOR EACH USER USAGE
    for i in range(len(users)):
        plt.plot(range(final_time),usage_evolution[i], label=users[i])

    plt.xlabel("Time in seconds")
    plt.ylabel("Usage ($Cores \cdot s$)")
    plt.title(f"Evolution of usage per user\n Trace: {file_name}")
    plt.legend()
    plt.savefig(f"{dir_path}/{file_name}_instant_usage_per_user.png",dpi=150)
    plt.clf()

if __name__ == "__main__":
    if len(sys.argv) == 1:
        print("You have to provide a file to read")
        sys.exit(1)

    file_path = sys.argv[1]
    file_name = file_path.split("/")[-1].split(".")[0] # pick the file name and remove its file type
    dir_path = file_path.rsplit("/",1)[0]
    print(f"Reading data from {file_path}...")
    data = pd.read_csv(file_path, sep=";")

    overall_start_time(data, file_name, dir_path)
    overall_requested_nodes(data, file_name, dir_path)
    overall_time_in_queue(data, file_name, dir_path)
    overall_2dhist_nodes_x_wait_time(data, file_name, dir_path)
    overall_submit_time(data, file_name, dir_path)
    overall_evolution_machine_usage(data, file_name, dir_path)
    user_start_time(data, file_name, dir_path)
    user_nodes_requested(data, file_name, dir_path)
    user_wait_time(data, file_name, dir_path)
    user_usage(data, file_name, dir_path)
